@@ -3,7 +3,7 @@
 # Author:  Boris Pek <tehnick-8@yandex.ru>
 # License: GPLv2 or later
 # Created: 2012-02-13
-# Updated: 2017-08-24
+# Updated: 2018-03-21
 # Version: N/A
 
 set -e
@@ -11,10 +11,6 @@ set -e
 export SNAPSHOTS_DIR="$(dirname $(realpath -s ${0}))"
 export MAIN_DIR="$(realpath -s ${SNAPSHOTS_DIR}/..)"
 
-MAIN_VERSION=1.2
-PSI_DEFINING_COMMIT=4b8a3473ee1de59f84627000cba462e05f8a9b84
-PATCHES_DEFINING_COMMIT=db1a4053ef7aae72ae819eb00eba47bae9530320
-
 PSI_URL=https://github.com/psi-im/psi.git
 PLUGINS_URL=https://github.com/psi-im/plugins.git
 PATCHES_URL=https://github.com/psi-plus/main.git
@@ -115,13 +111,13 @@ echo "Updating is required!";
 echo;
 
 cd "${SNAPSHOTS_DIR}"
-echo "Statring Psi+ update..."
+echo "Starting Psi+ update..."
 
 find . -type f | \
     grep -v "^\./\.git" | \
-    grep -v "^\./generate-single-repo.sh" | \
-    grep -v "^\./configure" | \
-    grep -v "^\./README" | \
+    grep -v "^\./generate-single-repo.sh$" | \
+    grep -v "^\./configure$" | \
+    grep -v "^\./README$" | \
     while read var; do rm "$var"; done
 find . -depth -type d -empty -exec rmdir {} \;
 echo "* Directory is cleaned."
@@ -134,12 +130,13 @@ for FILE in generate-single-repo.sh configure README; do
 done
 chmod uog+x generate-single-repo.sh configure
 
-mv "${SNAPSHOTS_DIR}/configure" "${MAIN_DIR}/configure"
-mv "${SNAPSHOTS_DIR}/README" "${MAIN_DIR}/README"
+cp -f "${SNAPSHOTS_DIR}/configure" "${MAIN_DIR}/configure"
+cp -f "${SNAPSHOTS_DIR}/README" "${MAIN_DIR}/README"
 rsync -a "${MAIN_DIR}/psi/" "${SNAPSHOTS_DIR}/" \
     --exclude=".git*" \
-    --exclude="^configure" \
-    --exclude="^README"
+    --exclude="/configure" \
+    --exclude="/README.md" \
+    --exclude="/README"
 mv "${MAIN_DIR}/configure" "${SNAPSHOTS_DIR}/configure"
 mv "${MAIN_DIR}/README" "${SNAPSHOTS_DIR}/README"
 echo "* Files from psi project are copied."
@@ -192,7 +189,7 @@ echo "* Skins from Psi+ project are copied."
 rsync -a "${MAIN_DIR}/resources/themes/" "${SNAPSHOTS_DIR}/themes/"
 echo "* Themes from Psi+ project are copied."
 
-cp "${MAIN_DIR}/main/changelog.txt" "${SNAPSHOTS_DIR}/ChangeLog"
+cp "${MAIN_DIR}/main/ChangeLog.Psi+.txt" "${SNAPSHOTS_DIR}/ChangeLog.Psi+.txt"
 echo "* ChangeLog from Psi+ project is copied."
 
 remove_trash
@@ -228,10 +225,15 @@ $(cd ${MAIN_DIR}/plugins    && git log -n1 --date=short --pretty=format:'%ad')
 $(cd ${MAIN_DIR}/resources  && git log -n1 --date=short --pretty=format:'%ad')"
 LAST_REVISION_DATE=$(echo "${REVISION_DATE_LIST}" | sort -r | head -n1)
 
-PSI_NUM="$(cd ${MAIN_DIR}/psi && git rev-list --count ${PSI_DEFINING_COMMIT}..HEAD)"
-PATCHES_NUM="$(cd ${MAIN_DIR}/main && git rev-list --count ${PATCHES_DEFINING_COMMIT}..HEAD)"
+PATCHES_VERSION="$(cd ${MAIN_DIR}/main && git describe --tags | cut -d - -f1)"
+PATCHES_NUM="$(cd ${MAIN_DIR}/main && git describe --tags | cut -d - -f2)"
+PSI_NUM="0"
 
-NEW_VER="${MAIN_VERSION}.$(expr ${PSI_NUM} + ${PATCHES_NUM})"
+if [ "$(cd ${MAIN_DIR}/psi && git tag | grep -x "^${PATCHES_VERSION}$" | wc -l)" = "1" ]; then
+    PSI_NUM="$(cd ${MAIN_DIR}/psi && git rev-list --count ${PATCHES_VERSION}..HEAD)"
+fi
+
+NEW_VER="${PATCHES_VERSION}.$(expr ${PSI_NUM} + ${PATCHES_NUM})"
 OLD_VER=$(cd "${SNAPSHOTS_DIR}" && git tag -l | sort -V | tail -n1)
 
 echo "OLD_VER = ${OLD_VER}"
@@ -253,7 +255,7 @@ It is based on:
 "
 echo "${COMMENT}"
 
-git cm -a -m "${COMMENT}" 2>&1 > /dev/null
+git commit -a -m "${COMMENT}" 2>&1 > /dev/null
 
 if [ "${NEW_VER}" != "${OLD_VER}" ]; then
     git tag "${NEW_VER}"
