@@ -63,6 +63,8 @@ namespace XMPP
         bool tls_supported, sasl_supported, bind_supported, compress_supported;
         bool tls_required;
         bool sm_supported;
+        bool session_supported;
+        bool session_required;
         QStringList sasl_mechs;
         QStringList compression_mechs;
         QStringList hosts;
@@ -71,15 +73,19 @@ namespace XMPP
     class BasicProtocol : public XmlProtocol
     {
     public:
-        // xmpp 1.0 error conditions
+        // xmpp 1.0 error conditions // rfc6120
         enum SASLCond {
-            Aborted,
-            IncorrectEncoding,
-            InvalidAuthzid,
-            InvalidMech,
-            MechTooWeak,
-            NotAuthorized,
-            TemporaryAuthFailure
+            Aborted,                    // server confirms auth abort
+            AccountDisabled,            // account temporrily disabled
+            CredentialsExpired,         // credential expired
+            EncryptionRequired,         // can't use mech without TLS
+            IncorrectEncoding,          // Incorrect encoding. should not happen
+            InvalidAuthzid,             // bad input JID
+            InvalidMech,                // bad mechanism
+            MalformedRequest,           // malformded request
+            MechTooWeak,                // can't use mech with this authzid
+            NotAuthorized,              // bad user, bad password, bad creditials
+            TemporaryAuthFailure,       // please try again later!
         };
         enum StreamCond {
             BadFormat,
@@ -96,6 +102,7 @@ namespace XMPP
             StreamNotAuthorized,
             PolicyViolation,
             RemoteConnectionFailed,
+            StreamReset,
             ResourceConstraint,
             RestrictedXml,
             SeeOtherHost,
@@ -161,6 +168,7 @@ namespace XMPP
         void sendStanza(const QDomElement &e);
         void sendDirect(const QString &s);
         void sendWhitespace();
+        void clearSendQueue();
         QDomElement recvStanza();
 
         // shutdown
@@ -174,6 +182,7 @@ namespace XMPP
         // error output
         int errCond;
         QString errText;
+        QHash<QString,QString> errLangText;
         QDomElement errAppSpec;
         QString otherHost;
 
