@@ -134,6 +134,7 @@ public:
     FileTransferManager *ftman;
     bool ftEnabled;
     QList<GroupChat> groupChatList;
+    EncryptionHandler *encryptionHandler;
 };
 
 
@@ -211,7 +212,7 @@ void Client::start(const QString &host, const QString &user, const QString &pass
     connect(pp, SIGNAL(subscription(Jid,QString,QString)), SLOT(ppSubscription(Jid,QString,QString)));
     connect(pp, SIGNAL(presence(Jid,Status)), SLOT(ppPresence(Jid,Status)));
 
-    JT_PushMessage *pm = new JT_PushMessage(rootTask());
+    JT_PushMessage *pm = new JT_PushMessage(rootTask(), d->encryptionHandler);
     connect(pm, SIGNAL(message(Message)), SLOT(pmMessage(Message)));
 
     JT_PushRoster *pr = new JT_PushRoster(rootTask());
@@ -657,6 +658,12 @@ void Client::send(const QString &str)
     static_cast<ClientStream*>(d->stream)->writeDirect(str);
 }
 
+/* drops any pending outgoing xml elements */
+void Client::clearSendQueue()
+{
+    d->stream->clearSendQueue();
+}
+
 bool Client::hasStream() const
 {
     return !!d->stream;
@@ -682,6 +689,11 @@ const ResourceList & Client::resourceList() const
     return d->resourceList;
 }
 
+bool Client::isSessionRequired() const
+{
+    return d->stream && !d->stream->old() && d->stream->streamFeatures().session_required;
+}
+
 QString Client::host() const
 {
     return d->host;
@@ -1035,7 +1047,7 @@ void Client::importRosterItem(const RosterItem &item)
     debug(dstr + str);
 }
 
-void Client::sendMessage(const Message &m)
+void Client::sendMessage(Message &m)
 {
     JT_Message *j = new JT_Message(rootTask(), m);
     j->go(true);
@@ -1154,6 +1166,16 @@ void Client::setCaps(const CapsSpec &s)
     d->caps = s;
 }
 
+void Client::setEncryptionHandler(EncryptionHandler *encryptionHandler)
+{
+    d->encryptionHandler = encryptionHandler;
+}
+
+EncryptionHandler *Client::encryptionHandler() const
+{
+    return d->encryptionHandler;
+}
+
 DiscoItem::Identity Client::identity() const
 {
     return d->identity;
