@@ -50,6 +50,7 @@ namespace XMPP {
     class Stream;
     class Task;
     class CapsManager;
+    class EncryptionHandler;
 }
 
 namespace XMPP
@@ -72,9 +73,11 @@ namespace XMPP
         QString streamBaseNS() const;
         const LiveRoster & roster() const;
         const ResourceList & resourceList() const;
+        bool isSessionRequired() const;
 
         void send(const QDomElement &);
         void send(const QString &);
+        void clearSendQueue();
 
         QString host() const;
         QString user() const;
@@ -83,7 +86,7 @@ namespace XMPP
         Jid jid() const;
 
         void rosterRequest(bool withGroupsDelimiter = true);
-        void sendMessage(const Message &);
+        void sendMessage(Message &);
         void sendSubscription(const Jid &, const QString &, const QString& nick = QString());
         void setPresence(const Status &);
 
@@ -108,6 +111,8 @@ namespace XMPP
         void setClientName(const QString &);
         void setClientVersion(const QString &);
         void setCaps(const CapsSpec &);
+        void setEncryptionHandler(EncryptionHandler *);
+        EncryptionHandler *encryptionHandler() const;
 
         void setIdentity(const DiscoItem::Identity &);
         DiscoItem::Identity identity() const;
