@@ -719,7 +719,9 @@ void JT_Presence::sub(const Jid &to, const QString &subType, const QString& nick
     tag = doc()->createElement("presence");
     tag.setAttribute("to", to.full());
     tag.setAttribute("type", subType);
-    if (!nick.isEmpty()) {
+    if (!nick.isEmpty() && (subType == QLatin1String("subscribe") || subType == QLatin1String("subscribed") ||
+                            subType == QLatin1String("unsubscribe") || subType == QLatin1String("unsubscribed")))
+    {
         QDomElement nick_tag = textTag(doc(),"nick",nick);
         nick_tag.setAttribute("xmlns","http://jabber.org/protocol/nick");
         tag.appendChild(nick_tag);
@@ -773,7 +775,8 @@ bool JT_PushPresence::take(const QDomElement &e)
             getErrorFromElement(e, client()->stream().baseNS(), &code, &str);
             p.setError(code, str);
         }
-        else if(type == "subscribe" || type == "subscribed" || type == "unsubscribe" || type == "unsubscribed") {
+        else if(type == QLatin1String("subscribe") || type == QLatin1String("subscribed") ||
+                type == QLatin1String("unsubscribe") || type == QLatin1String("unsubscribed")) {
             QString nick;
             QDomElement tag = e.firstChildElement("nick");
             if (!tag.isNull() && tag.attribute("xmlns") == "http://jabber.org/protocol/nick") {
@@ -924,12 +927,11 @@ static QDomElement oldStyleNS(const QDomElement &e)
     return i;
 }
 
-JT_Message::JT_Message(Task *parent, const Message &msg)
-:Task(parent)
+JT_Message::JT_Message(Task *parent, Message &msg)
+:Task(parent), m(msg)
 {
-    m = msg;
-    if (m.id().isEmpty())
-        m.setId(id());
+    if (msg.id().isEmpty())
+        msg.setId(id());
 }
 
 JT_Message::~JT_Message()
@@ -938,9 +940,18 @@ JT_Message::~JT_Message()
 
 void JT_Message::onGo()
 {
+
     Stanza s = m.toStanza(&(client()->stream()));
     QDomElement e = oldStyleNS(s.element());
-    send(e);
+
+    auto encryptionHandler = client()->encryptionHandler();
+    bool wasEncrypted = encryptionHandler && encryptionHandler->encryptMessageElement(e);
+    m.setWasEncrypted(wasEncrypted);
+
+    // if the element is null, then the encryption is happening asynchronously
+    if (!e.isNull()) {
+        send(e);
+    }
     setSuccess();
 }
 
@@ -948,13 +959,21 @@ void JT_Message::onGo()
 //----------------------------------------------------------------------------
 // JT_PushMessage
 //----------------------------------------------------------------------------
-JT_PushMessage::JT_PushMessage(Task *parent)
+class JT_PushMessage::Private {
+public:
+    EncryptionHandler *m_encryptionHandler;
+};
+
+JT_PushMessage::JT_PushMessage(Task *parent, EncryptionHandler *encryptionHandler)
 :Task(parent)
 {
+    d = new Private;
+    d->m_encryptionHandler = encryptionHandler;
 }
 
 JT_PushMessage::~JT_PushMessage()
 {
+    delete d;
 }
 
 bool JT_PushMessage::take(const QDomElement &e)
@@ -963,6 +982,13 @@ bool JT_PushMessage::take(const QDomElement &e)
         return false;
 
     QDomElement e1 = e;
+
+    bool wasEncrypted = d->m_encryptionHandler != nullptr && d->m_encryptionHandler->decryptMessageElement(e1) && !e1.isNull();
+    if (wasEncrypted && e1.isNull()) {
+        // The message was processed, but has to be discarded for some reason
+        return true;
+    }
+
     QDomElement forward;
     Message::CarbonDir cd = Message::NoCarbon;
 
@@ -1011,6 +1037,7 @@ bool JT_PushMessage::take(const QDomElement &e)
         m.setForwardedFrom(fromJid);
         m.setCarbonDirection(cd);
     }
+    m.setWasEncrypted(wasEncrypted);
 
     emit message(m);
     return true;
