@@ -29,6 +29,7 @@
 #include "xmpp_vcard.h"
 #include "xmpp_discoinfotask.h"
 #include "xmpp_subsets.h"
+#include "xmpp_encryptionhandler.h"
 
 namespace XMPP
 {
@@ -191,7 +192,7 @@ namespace XMPP
     {
         Q_OBJECT
     public:
-        JT_Message(Task *parent, const Message &);
+        JT_Message(Task *parent, Message &);
         ~JT_Message();
 
         void onGo();
@@ -207,7 +208,7 @@ namespace XMPP
     {
         Q_OBJECT
     public:
-        JT_PushMessage(Task *parent);
+        JT_PushMessage(Task *parent, EncryptionHandler *encryptionHandler = nullptr);
         ~JT_PushMessage();
 
         bool take(const QDomElement &);
