@@ -34,6 +34,7 @@
 #include <QFile>
 #include <QBuffer>
 #include <QPainter>
+#include <QImageReader>
 
 #include "xmpp_xmlcommon.h"
 #include "xmpp_vcard.h"
@@ -192,7 +193,7 @@ public:
         }
 
         QString hash(_hash);
-        if (newData != data || hash.isEmpty()) {
+        if (hash.isEmpty()) { // ignore "newData != data ||" since we need hashing by original hash
             hash = QString::fromLatin1(QCryptographicHash::hash(newData, QCryptographicHash::Sha1).toHex());
         }
 
@@ -328,7 +329,7 @@ protected:
 
 private:
     AvatarCache()
-        : FileCache(AvatarFactory::getCacheDir(), QApplication::instance())
+        : FileCache(AvatarFactory::getCacheDir())
     {
         updateJids();
     }
@@ -656,7 +657,7 @@ QPixmap AvatarFactory::getAvatar(const Jid& _jid)
         }
         QByteArray data = vcard.photo();
         if (AvatarCache::instance()->setIcon(AvatarCache::VCardType, bareJid, data) != AvatarCache::NoData) {
-            img = QImage::fromData(data);
+            img = QImage::fromData(AvatarCache::instance()->icons(bareJid).avatar->data()); // from scaled avatar
         }
     }
 
@@ -741,15 +742,18 @@ QPixmap AvatarFactory::getMucAvatar(const Jid& _jid)
 
     auto icons = AvatarCache::instance()->icons(fullJid);
     QByteArray data;
-    if (icons.avatar) {
-        data = icons.avatar->data();
-    } else {
+    if (!icons.avatar) {
         auto vcard = VCardFactory::instance()->mucVcard(_jid);
         if (vcard.isNull() || vcard.photo().isNull()) {
             return QPixmap();
         }
         data = vcard.photo();
         AvatarCache::instance()->setIcon(AvatarCache::VCardType, _jid.full(), data);
+        icons = AvatarCache::instance()->icons(fullJid); // should return scaled copy
+    }
+
+    if (icons.avatar) {
+        data = icons.avatar->data();
     }
 
     // for mucs icons.avatar is always made of vcard and anything else is not supported. at least for now.
@@ -972,17 +976,34 @@ void AvatarFactory::itemPublished(const Jid& jid, const QString& n, const PubSub
     }
     else if (n == PEP_AVATAR_METADATA_NS) {
         QString hash = item.id().toLower();
+        if (hash.size() < 40) {
+            return; // doesn't look like sha1 hash. just ignore it
+        }
 
         if (item.payload().tagName() == QLatin1String(PEP_AVATAR_METADATA_TN) && item.payload().firstChildElement().isNull()) {
             // user wants to stop publishing avatar
             // previously we used "stop" element. now specs are changed
             result = AvatarCache::instance()->removeIcon(AvatarCache::AvatarType, jidFull);
         } else {
-            result = cache->appendUser(item.id(), AvatarCache::AvatarType,
-                                            jidFull);
-            if (result == AvatarCache::NoData) {
-                d->pa_->pepManager()->get(jid, PEP_AVATAR_DATA_NS, hash);
-                return;
+            auto mimes = QImageReader::supportedMimeTypes();
+
+            for (QDomElement e = item.payload().firstChildElement(QLatin1String("info")); !e.isNull(); e = e.nextSiblingElement(QLatin1String("info"))) {
+                if (!mimes.contains(e.attribute(QLatin1String("type")).toLower().toLatin1())) {
+                    continue; // unsupported mime
+                }
+                if (!e.attribute(QLatin1String("url")).isEmpty()) {
+                    continue; // web avatars are not currently supported. TODO but their support is highly expected
+                }
+                if (e.attribute(QLatin1String("id")) != hash) {
+                    continue; // that's something totally unexpected
+                }
+                // found in-band png (by xep84 hash is for png) avatar. So we can make request
+                result = cache->appendUser(item.id(), AvatarCache::AvatarType, jidFull);
+                if (result == AvatarCache::NoData) {
+                    d->pa_->pepManager()->get(jid, PEP_AVATAR_DATA_NS, hash);
+                    return;
+                }
+                break;
             }
         }
     }
