@@ -56,6 +56,7 @@
 #include "iconwidget.h"
 #include "textutil.h"
 #include "xmpp_message.h"
+#include "xmpp_caps.h"
 #include "xmpp_htmlelement.h"
 #include "fancylabel.h"
 #include "msgmle.h"
@@ -111,8 +112,12 @@ ChatDlg::ChatDlg(const Jid& jid, PsiAccount* pa, TabManager* tabManager)
 
     status_ = -1;
 
-    historyState = false;
-    preloadHistory();
+    if (!pa->findGCContact(jid) || ((pa->edb()->features() & EDB::PrivateContacts) != 0)) {
+        historyState = false;
+        preloadHistory();
+    }
+    else
+        historyState = true;
 
     autoSelectContact_ = false;
     if (PsiOptions::instance()->getOption("options.ui.chat.default-jid-mode").toString() == "auto") {
@@ -124,6 +129,9 @@ ChatDlg::ChatDlg(const Jid& jid, PsiAccount* pa, TabManager* tabManager)
 
     // Message events
     contactChatState_ = XMPP::StateNone;
+    if ((PsiOptions::instance()->getOption("options.messages.send-composing-events-at-start").toBool()) && (account()->client()->capsManager()->features(jid).canChatState())) {
+        contactChatState_ = XMPP::StateActive;
+    }
     lastChatState_ = XMPP::StateNone;
     sendComposingEvents_ = false;
     isComposing_ = false;
@@ -277,9 +285,9 @@ bool ChatDlg::readyToHide()
 
     // Reset 'contact is composing' & cancel own composing event
     resetComposing();
-    setChatState(StateGone);
+    setChatState(XMPP::StateGone);
     if (contactChatState_ == XMPP::StateComposing || contactChatState_ == XMPP::StateInactive) {
-        setContactChatState(StatePaused);
+        setContactChatState(XMPP::StatePaused);
     }
 
     if (pending_ > 0) {
@@ -308,7 +316,7 @@ void ChatDlg::hideEvent(QHideEvent* e)
 {
     if (isMinimized()) {
         resetComposing();
-        setChatState(StateInactive);
+        setChatState(XMPP::StateInactive);
     }
     TabbableWidget::hideEvent(e);
 }
@@ -703,7 +711,7 @@ QString ChatDlg::desiredCaption() const
     if (contactChatState_ == XMPP::StateComposing) {
         cap = tr("%1 (Composing ...)").arg(cap);
     }
-    else if (contactChatState_ == XMPP::StateInactive) {
+    else if (contactChatState_ == XMPP::StateInactive || contactChatState_ == XMPP::StateGone) {
         cap = tr("%1 (Inactive)").arg(cap);
     }
 
@@ -936,7 +944,7 @@ void ChatDlg::appendMessage(const Message &m, bool local)
                 MessageView::System
         ));
         if (!local) {
-            setPGPEnabled(encEnabled);
+            setPGPEnabled(encEnabled && account()->hasPGP());
             if (!encEnabled) {
                 // enable warning
                 warnSend_ = true;
@@ -1249,12 +1257,14 @@ void ChatDlg::chatEditCreated()
 
 TabbableWidget::State ChatDlg::state() const
 {
-    TabbableWidget::State state = TabbableWidget::StateNone;
+    TabbableWidget::State state = TabbableWidget::State::None;
     if(contactChatState_ == XMPP::StateComposing) {
-        state = TabbableWidget::StateComposing;
+        state = TabbableWidget::State::Composing;
     }
     else if(unreadMessageCount()) {
-        state = TabbableWidget::StateHighlighted;
+        state = TabbableWidget::State::Highlighted;
+    } else if (contactChatState_ == XMPP::StateInactive || contactChatState_ == XMPP::StateGone) {
+        state = TabbableWidget::State::Inactive;
     }
 
     return state;
