@@ -409,23 +409,25 @@ ChatView::~ChatView()
 void ChatView::init()
 {
     Theme curTheme = d->themeProvider()->current();
-    if (curTheme.state() != Theme::Loaded) {
+    //qDebug() << "Init chatview with theme" << curTheme.name();
+    if (curTheme.state() != Theme::State::Loaded) {
         qDebug("ChatView theme is not loaded. this is fatal");
         return;
     }
-    d->theme = curTheme;// TODO rewrite this pointer magic
+    d->theme = curTheme;
 
 #ifndef WEBENGINE
     ((ChatViewPage*)d->webView->page())->setCVPrivate(d.data());
 #endif
     d->jsObject->init(d->theme);
 
+    /* this commented out code is broken. try to fix it if you can.
     bool tbg = d->jsObject->isTransparentBackground();
     QWidget *w = this;
     while (w) {
         w->setAttribute(Qt::WA_TranslucentBackground, tbg);
         w = w->parentWidget();
-    }
+    }*/
 }
 
 void ChatView::setEncryptionEnabled(bool enabled)
