@@ -127,17 +127,17 @@ bool ChatViewThemePrivate::load(std::function<void(bool)> loadCallback)
         return false;
     }
 
-    if (state == Theme::Loaded) {
+    if (state == Theme::State::Loaded) {
         loadCallback(true);
         return true;
     }
     loadCallbacks.push_back(loadCallback);
-    if (state == Theme::Loading) {
+    if (state == Theme::State::Loading) {
         return true;
     }
 
     qDebug("Starting loading \"%s\" theme at \"%s\" for %s", qPrintable(id), qPrintable(filepath), isMuc()? "muc": "chat");
-    state = Theme::Loading;
+    state = Theme::State::Loading;
     if (jsUtil.isNull()) {
         jsLoader.reset(new ChatViewJSLoader(this));
         jsUtil.reset(new ChatViewThemeJSUtil(this));
@@ -230,6 +230,8 @@ QVariantMap ChatViewThemePrivate::loadFromCacheMulti(const QVariantList &list)
     for (auto &item : list) {
         QString key = item.toString();
         ret[key] = cache.value(key);
+        //if (key.endsWith("html"))
+        //    qDebug() << "Loaded from cache" << key << ret[key].toString().left(50) << "priv=" << (void*)this;
     }
     return ret;
 }
@@ -266,15 +268,15 @@ bool ChatViewThemePrivate::applyToSession(ChatViewThemeSession *session)
     }
 
     QWebChannel *channel = page->webChannel();
-    if (!channel) {
-        channel = new QWebChannel(session->webView());
-
-        channel->registerObject(QLatin1String("srvUtil"), new ChatViewThemeJSUtil(this, session->webView()));
-        channel->registerObject(QLatin1String("srvSession"), session);
-
-        page->setWebChannel(channel);
-        // channel is kept on F5 but all objects are cleared, so will be added later
+    QObject *oldUtil = nullptr;
+    if (channel) {
+        oldUtil = channel->registeredObjects()[QLatin1String("srvUtil")];
+        oldUtil->deleteLater();
     }
+    channel = new QWebChannel(session->webView());
+    channel->registerObject(QLatin1String("srvUtil"), new ChatViewThemeJSUtil(this, session->webView()));
+    channel->registerObject(QLatin1String("srvSession"), session);
+    page->setWebChannel(channel);
 
     ChatViewThemeProvider *cvProvider = static_cast<ChatViewThemeProvider*>(provider);
 
@@ -345,8 +347,24 @@ bool ChatViewThemePrivate::applyToSession(ChatViewThemeSession *session)
             }
             return true;
         } else {
-            QByteArray data = Theme(this).loadData(httpRelPath + path);
-            if (!data.isNull()) {
+            bool loaded;
+            QByteArray data = Theme(this).loadData(httpRelPath + path, &loaded);
+            if (loaded) {
+                if (!data.isNull() && path.endsWith(QLatin1String(".tiff"))) {
+                    // seems like we are loading tiff image which is supported by safari only.
+                    // let's convert it
+                    QImage image(QImage::fromData(data));
+                    QByteArray ba;
+                    QBuffer buffer(&ba);
+                    buffer.open(QIODevice::WriteOnly);
+                    image.save(&buffer, "PNG");
+                    if (!ba.isNull()) {
+                        data = ba;
+                    }
+                }
+                if (path.endsWith(QLatin1String(".css"))) {
+                    res->headers().insert("Content-Type", "text/css;charset=utf-8");
+                }
                 res->setStatusCode(qhttp::ESTATUS_OK);
                 res->end(data);
                 return true;
@@ -443,7 +461,7 @@ void ChatViewJSLoader::unregisterSession(const QString &sessId)
 void ChatViewJSLoader::_callFinishLoadCalbacks()
 {
     for (auto &cb : theme->loadCallbacks) {
-        cb(theme->state == Theme::Loaded);
+        cb(theme->state == Theme::State::Loaded);
     }
 }
 
@@ -478,7 +496,7 @@ void ChatViewJSLoader::setMetaData(const QVariantMap &map)
 void ChatViewJSLoader::finishThemeLoading()
 {
     qDebug("%s theme is successfully loaded for %s", qPrintable(theme->id), theme->isMuc()? "muc": "chat");
-    Theme(theme).setState(Theme::Loaded);
+    Theme(theme).setState(Theme::State::Loaded);
 #ifdef WEBENGINE
     _callFinishLoadCalbacks();
 #else
@@ -489,7 +507,7 @@ void ChatViewJSLoader::finishThemeLoading()
 void ChatViewJSLoader::errorThemeLoading(const QString &error)
 {
     _loadError = error;
-    Theme(theme).setState(Theme::NotLoaded);
+    Theme(theme).setState(Theme::State::NotLoaded);
 #ifdef WEBENGINE
     _callFinishLoadCalbacks();
 #else
@@ -516,9 +534,12 @@ void ChatViewJSLoader::saveFilesToCache(const QVariantMap &map)
 {
     auto it = map.constBegin();
     while (it != map.constEnd()) {
-        QByteArray ba = Theme(theme).loadData(it.value().toString());
-        if (!ba.isNull()) {
+        bool loaded;
+        QByteArray ba = Theme(theme).loadData(it.value().toString(), &loaded);
+        if (loaded) {
             theme->cache.insert(it.key(), QString::fromUtf8(ba));
+            //qDebug() << "Caching" << it.value() << "from" << Theme(theme).filePath()
+            //         << theme->cache[it.key()] << "priv=" << (void*)theme;
         }
         ++it;
     }
@@ -575,8 +596,13 @@ QString ChatViewJSLoader::getFileContents(const QString &name) const
 
 QString ChatViewJSLoader::getFileContentsFromAdapterDir(const QString &name) const
 {
-    QString adapterPath = theme->provider->themePath(QLatin1String("chatview/") + theme->id.split('/').first());
-    QFile file(adapterPath + "/" + name);
+    QString relPath = QLatin1String("chatview/") + theme->id.split('/').first() + QLatin1Char('/') + name;
+    QString filePath = theme->provider->themePath(relPath);
+    if (filePath.isEmpty()) {
+        qDebug("%s is not found", qPrintable(relPath));
+        return QString();
+    }
+    QFile file(filePath);
     if (file.open(QIODevice::ReadOnly)) {
         QByteArray result = file.readAll();
         file.close();
