@@ -73,6 +73,7 @@
 #include "xmpp_tasks.h"
 #include "xmpp_caps.h"
 #include "iconaction.h"
+#include "pixmapratiolabel.h"
 #include "psitooltip.h"
 #include "avatars.h"
 #include "psioptions.h"
@@ -247,6 +248,8 @@ public:
     int hPending; // highlight pending
     bool connecting;
     bool alert;
+    bool gcSelfPresenceSupported = false;
+    bool gcSelfAvatarRequested = false; // when self presence is not supported
 
     QStringList hist;
     int histAt;
@@ -1061,7 +1064,6 @@ GCMainDlg::GCMainDlg(PsiAccount *pa, const Jid &j, TabManager *tabManager)
     connect(disco, SIGNAL(finished()), SLOT(discoInfoFinished()));     // but we need this just for name for now.
     disco->get(jid());                                             // From other side we could provide the name outside.
     disco->go(true);
-    VCardFactory::instance()->getVCard(jid(), account()->client()->rootTask(), this, SLOT(updateGCVCard()), true);
 
     setLooks();
     setToolbuttons();
@@ -1341,9 +1343,29 @@ void GCMainDlg::discoInfoFinished()
     }
 }
 
+// this one is called as result of avatarChanged event from avatar factory or
+void GCMainDlg::setMucSelfAvatar()
+{
+    bool enabled = PsiOptions::instance()->getOption("options.ui.chat.avatars.show").toBool();
+    QPixmap p;
+    if (enabled) {
+        p = account()->avatarFactory()->getAvatar(jid().withResource(QString()));
+        enabled = !p.isNull();
+    }
+    ui_.lblAvatar->setVisible(enabled);
+    if (enabled) {
+        ui_.lblAvatar->setResizePolicy(PixmapRatioLabel::Policy::FitVertical);
+        ui_.lblAvatar->setMaxPixmapSize(QSize(64,64) * devicePixelRatio());
+        ui_.lblAvatar->setPixmap(p);
+    } else {
+        ui_.lblAvatar->resize(0, 0);
+    }
+}
+
 void GCMainDlg::updateGCVCard()
 {
     const VCard vcard = VCardFactory::instance()->vcard(jid());
+    QPixmap avatar;
     if (vcard) {
         d->vcardMucName = vcard.nickName();
         if (d->vcardMucName.isEmpty()) {
@@ -1352,14 +1374,9 @@ void GCMainDlg::updateGCVCard()
         if (d->mucNameSource >= Private::TitleVCard) {
             updateMucName();
         }
-        QImage avatar = QImage::fromData(vcard.photo());
-        if (!avatar.isNull()) {
-            ui_.lblAvatar->show();
-            ui_.lblAvatar->setPixmap(QPixmap::fromImage(avatar).scaled(ui_.lblAvatar->minimumSize(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
-            return;
-        }
+        avatar.loadFromData(vcard.photo());
     }
-    ui_.lblAvatar->hide();
+    //setMucSelfAvatar(avatar);
 }
 
 void MiniCommand_Depreciation_Message(const QString &old,const QString &newCmd, QString &line1, QString &line2) {
@@ -1749,6 +1766,13 @@ void GCMainDlg::mucKickMsgHelper(const QString &nick, const Status &s, const QSt
     appendSysMsg(message);
 }
 
+// presence from muc itself (resource is empty)
+void GCMainDlg::gcSelfPresence(const Status &s)
+{
+    d->gcSelfPresenceSupported = true;
+    account()->avatarFactory()->statusUpdate(jid().withResource(QString()), s);
+}
+
 void GCMainDlg::presence(const QString &nick, const Status &s)
 {
     if(s.hasError()) {
@@ -1770,6 +1794,11 @@ void GCMainDlg::presence(const QString &nick, const Status &s)
 
     bool isSelf = (nick == d->self);
     if (isSelf) {
+        if (!d->gcSelfPresenceSupported && !d->gcSelfAvatarRequested) {
+            d->gcSelfAvatarRequested = true;
+            VCardFactory::instance()->getVCard(jid(), account()->client()->rootTask(), this, SLOT(updateGCVCard()), true);
+        }
+
         if(s.isAvailable())
             setStatusTabIcon(s.type());
         UserListItem* u = account()->find(d->dlg->jid().bare());
@@ -1928,6 +1957,14 @@ void GCMainDlg::presence(const QString &nick, const Status &s)
                          tr("%1 has been banned by %2"));
             suppressDefault = true;
         }
+        if (s.getMUCStatuses().contains(333)) {
+            mucKickMsgHelper(nick, s, nickJid, tr("Removed"),
+                         tr("You have been removed from the room due to technical problem"),
+                         tr("You have been removed from the room by %1 due to technical problem"),
+                         tr("%1 has been removed from the room due to technical problem"),
+                         tr("%1 has been removed from the room by %2 due to technical problem"));
+            suppressDefault = true;
+        } else // 333 and 307 can come together. so "else" is here
         if (s.getMUCStatuses().contains(307)) {
             // Kick
             mucKickMsgHelper(nick, s, nickJid, tr("Kicked"), tr("You have been kicked from the room"),
@@ -1994,6 +2031,7 @@ void GCMainDlg::avatarUpdated(const Jid &jid_)
     if(jid_.compare(jid(), false)) {
         if (jid_.resource().isEmpty()) {
             ui_.log->updateAvatar(jid_, ChatViewCommon::RemoteParty);
+            setMucSelfAvatar();
             return;
         }
         d->usersModel->updateAvatar(jid_.resource());
@@ -2021,7 +2059,10 @@ void GCMainDlg::message(const Message &_m, const PsiEvent::Ptr &e)
     }
     if (m.getMUCStatuses().contains(104)) {
         // new MUC vcard available
-        VCardFactory::instance()->getVCard(jid(), account()->client()->rootTask(), this, SLOT(updateGCVCard()), true);
+        if (!d->gcSelfPresenceSupported) {
+            // we had to handle avatar hash from presence already
+            VCardFactory::instance()->getVCard(jid(), account()->client()->rootTask(), this, SLOT(updateGCVCard()), true);
+        }
     }
 
     PsiOptions *options = PsiOptions::instance();
@@ -2297,6 +2338,7 @@ void GCMainDlg::setLooks()
     ui_.lv_users->setVerticalScrollBarPolicy(PsiOptions::instance()->getOption("options.ui.muc.userlist.disable-scrollbar").toBool() ?
                                                  Qt::ScrollBarAlwaysOff : Qt::ScrollBarAsNeeded);
     ui_.lv_users->setLooks();
+    setMucSelfAvatar();
 }
 
 void GCMainDlg::setMargins()
@@ -2524,7 +2566,7 @@ void GCMainDlg::chatEditCreated()
 
 TabbableWidget::State GCMainDlg::state() const
 {
-    return d->hPending ? TabbableWidget::StateHighlighted : TabbableWidget::StateNone;
+    return d->hPending ? TabbableWidget::State::Highlighted : TabbableWidget::State::None;
 }
 
 int GCMainDlg::unreadMessageCount() const
@@ -2564,6 +2606,11 @@ void GCMainDlg::resizeEvent(QResizeEvent *e)
     QTimer::singleShot(0, this, SLOT(horizSplitterMoved()));
 }
 
+QStringList GCMainDlg::mucRosterContent() const
+{
+    return d->usersModel->nickList();
+}
+
 void GCMainDlg::sendButtonMenu()
 {
     SendButtonTemplatesMenu* menu = getTemplateMenu();
