@@ -323,10 +323,15 @@ void PsiMain::sessionQuit(int x)
 void PsiMain::bail()
 {
     if(pcon) {
-        delete pcon;
-        pcon = 0;
+        pcon->gracefulDeinit([this](){
+            pcon->deleteLater();
+            pcon = 0;
+            quit();
+        });
+    }
+    else {
+        quit();
     }
-    quit();
 }
 
 void PsiMain::saveSettings()
@@ -549,6 +554,9 @@ int main(int argc, char *argv[])
     QApplication::addLibraryPath(ApplicationInfo::resourcesDir());
     QApplication::addLibraryPath(ApplicationInfo::homeDir(ApplicationInfo::DataLocation));
     QApplication::setQuitOnLastWindowClosed(false);
+#if QT_VERSION >= 0x051000
+    QCoreApplication::setAttribute( Qt::AA_DisableWindowContextHelpButton, true );
+#endif
 
 #ifdef Q_OS_MAC
     QDir dir(QApplication::applicationDirPath());
