@@ -997,7 +997,7 @@ void MainWin::buildGeneralMenu(QMenu* menu)
 
 void MainWin::actReadmeActivated ()
 {
-    ShowTextDlg* w = new ShowTextDlg(":/README");
+    ShowTextDlg* w = new ShowTextDlg(":/README.html", true);
     w->setWindowTitle(CAP(tr("ReadMe")));
     w->show();
 }
@@ -1055,6 +1055,12 @@ void MainWin::actAboutQtActivated ()
 
 void MainWin::actAboutPsiMediaActivated ()
 {
+    if (!PsiMedia::isSupported()) {
+        QMessageBox::warning(this, tr("psimedia is unavailable"),
+                             tr("PsiMedia plugin is not loaded or not initialized.<br>"
+                                "Please check <a href=\"https://github.com/psi-im/psimedia\">https://github.com/psi-im/psimedia</a> for more details."));
+        return;
+    }
     QString creditText = PsiMedia::creditText();
     QString gstVersion = extract_gst_version(creditText);
 
@@ -1571,7 +1577,7 @@ void MainWin::toggleVisible(bool fromTray)
         bool hidden = false;
 #ifdef Q_OS_WIN
         if (fromTray) {
-            int timeout = 300;
+            unsigned timeout = 300;
             if (PsiOptions::instance()->getOption("options.ui.systemtray.use-double-click").toBool())
                 timeout += qApp->doubleClickInterval();
             hidden = isHidden() || (GetTickCount() - d->deactivationTickCount > timeout);
