@@ -32,6 +32,8 @@
 #include "eventcreatinghost.h"
 #include "contactinfoaccessinghost.h"
 #include "soundaccessinghost.h"
+#include "encryptionsupport.h"
+#include "pluginaccessinghost.h"
 
 class QWidget;
 class QPluginLoader;
@@ -41,12 +43,12 @@ class IqNamespaceFilter;
 
 class PluginHost: public QObject, public StanzaSendingHost, public IqFilteringHost, public OptionAccessingHost, public ShortcutAccessingHost, public IconFactoryAccessingHost,
     public ActiveTabAccessingHost, public ApplicationInfoAccessingHost, public AccountInfoAccessingHost, public PopupAccessingHost, public ContactStateAccessingHost
-    , public PsiAccountControllingHost, public EventCreatingHost, public ContactInfoAccessingHost, public SoundAccessingHost
+    , public PsiAccountControllingHost, public EventCreatingHost, public ContactInfoAccessingHost, public SoundAccessingHost, public EncryptionSupport, public PluginAccessingHost
 {
     Q_OBJECT
     Q_INTERFACES(StanzaSendingHost IqFilteringHost OptionAccessingHost ShortcutAccessingHost IconFactoryAccessingHost
         ActiveTabAccessingHost ApplicationInfoAccessingHost AccountInfoAccessingHost PopupAccessingHost ContactStateAccessingHost
-        PsiAccountControllingHost EventCreatingHost ContactInfoAccessingHost SoundAccessingHost)
+        PsiAccountControllingHost EventCreatingHost ContactInfoAccessingHost SoundAccessingHost EncryptionSupport PluginAccessingHost)
 
 public:
     PluginHost(PluginManager* manager, const QString& pluginFile);
@@ -78,7 +80,6 @@ public:
     // for StanzaFilter and IqNamespaceFilter
     bool incomingXml(int account, const QDomElement& e);
     bool outgoingXml(int account, QDomElement &e);
-    bool stanzaWasEncrypted(const QString &stanzaId);
 
     // for EventFilter
     bool processEvent(int account, QDomElement& e);
@@ -159,6 +160,8 @@ public:
     QString status(int account, const QString& jid);
     QString statusMessage(int account, const QString& jid);
     QStringList resources(int account, const QString& jid);
+    QString realJid(int account, const QString& jid);
+    QStringList mucNicks(int account, const QString& mucJid);
 
     //ContactStateAccessor
     bool setActivity(int account, const QString& Jid, QDomElement xml);
@@ -200,13 +203,21 @@ public:
     void setStatus(int account, const QString& status, const QString& statusMessage);
 
     bool appendSysMsg(int account, const QString& jid, const QString& message);
-    bool appendMsg(int account, const QString& jid, const QString& message, const QString& id);
+    bool appendMsg(int account, const QString& jid, const QString& message, const QString& id, bool wasEncrypted);
 
     void createNewEvent(int account, const QString& jid, const QString& descr, QObject *receiver, const char* slot);
     void createNewMessageEvent(int account, QDomElement const &element);
 
     void playSound(const QString& fileName);
 
+    // EncryptionSupport
+    bool decryptMessageElement(int account, QDomElement &message);
+    bool encryptMessageElement(int account, QDomElement &message);
+
+    // PluginAccessingHost
+
+    QObject* getPlugin(const QString &name);
+
 private:
     PluginManager* manager_;
     QPointer<QObject> plugin_;
