@@ -33,6 +33,7 @@
 #include "popupmanager.h"
 
 #include "chatdlg.h"
+#include "groupchatdlg.h"
 
 
 //TODO(mck)
@@ -210,14 +211,31 @@ void PluginManager::optionChanged(const QString& option)
     foreach (PluginHost* plugin, pluginByFile_.values()) {
         plugin->optionChanged(option);
         if(pluginOpt && plugin->shortName() == shortName) {
+            bool shouldUpdateFeatures;
             if(PsiOptions::instance()->getOption(option).toBool()) {
                 plugin->enable();
+                shouldUpdateFeatures = !plugin->pluginFeatures().isEmpty();
             }
             else {
+                shouldUpdateFeatures = !plugin->pluginFeatures().isEmpty();
                 if(optionsWidget_)
                     delete optionsWidget_;
                 plugin->unload();
             }
+            if (shouldUpdateFeatures) {
+                updateFeatures();
+            }
+        }
+    }
+}
+
+void PluginManager::updateFeatures()
+{
+    for (int acc_id = 0; accountIds_.isValidRange(acc_id); ++acc_id) {
+        PsiAccount *pa = accountIds_.account(acc_id);
+        pa->updateFeatures();
+        if (pa->isConnected()) {
+            pa->setStatusActual(pa->status());
         }
     }
 }
@@ -437,16 +455,6 @@ void PluginManager::processOutgoingStanza(PsiAccount* account, QDomElement &stan
     }
 }
 
-bool PluginManager::stanzaWasEncrypted(const QString &stanzaId)
-{
-    foreach (PluginHost* host, pluginByFile_.values()) {
-        if (host->stanzaWasEncrypted(stanzaId)) {
-            return true;
-        }
-    }
-    return false;
-}
-
 /**
  * Notify to plugins that an account will go offline now.
  */
@@ -901,11 +909,16 @@ bool PluginManager::appendSysMsg(int account, const QString& jid, const QString&
             chatDlg->appendSysMsg(message);
             return true;
         }
+        auto gcDlg = acc->findDialog<GCMainDlg*>(jid);
+        if(gcDlg) {
+            gcDlg->appendSysMsg(message);
+            return true;
+        }
     }
     return false;
 }
 
-bool PluginManager::appendMsg(int account, const QString& jid, const QString& message, const QString& id)
+bool PluginManager::appendMsg(int account, const QString& jid, const QString& message, const QString& id, bool wasEncrypted)
 {
     PsiAccount *acc = accountIds_.account(account);
     if(acc) {
@@ -922,6 +935,7 @@ bool PluginManager::appendMsg(int account, const QString& jid, const QString& me
             msg.setMessageReceipt(ReceiptRequest);
             msg.setId(id);
             msg.setTimeStamp(QDateTime::currentDateTime(), true);
+            msg.setWasEncrypted(wasEncrypted);
             chatDlg->appendMessage(msg, true);
             return true;
         }
@@ -1049,6 +1063,48 @@ QStringList PluginManager::resources(int account, const QString& jid) const
     return l;
 }
 
+QString PluginManager::realJid(int account, const QString& jid) const
+{
+    PsiAccount *acc = accountIds_.account(account);
+    if (acc) {
+      Jid realJid = acc->realJid(XMPP::Jid(jid));
+      return realJid.isNull() ? jid : realJid.full();
+    }
+    return jid;
+}
+
+QStringList PluginManager::mucNicks(int account, const QString &mucJid) const
+{
+    PsiAccount *acc = accountIds_.account(account);
+    if (acc) {
+        auto gcDlg = acc->findDialog<GCMainDlg*>(mucJid);
+        if(gcDlg) {
+            return gcDlg->mucRosterContent();
+        }
+    }
+    return {};
+}
+
+bool PluginManager::decryptMessageElement(PsiAccount *account, QDomElement &message) const
+{
+    foreach (PluginHost* host, pluginByFile_.values()) {
+        if (host->decryptMessageElement(accountIds_.id(account), message)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+bool PluginManager::encryptMessageElement(PsiAccount *account, QDomElement &message) const
+{
+    foreach (PluginHost* host, pluginByFile_.values()) {
+        if (host->encryptMessageElement(accountIds_.id(account), message)) {
+            return true;
+        }
+    }
+    return false;
+}
+
 int AccountIds::appendAccount(PsiAccount *acc)
 {
     int id = -1;
