@@ -73,7 +73,6 @@ public:
     bool processMessage(PsiAccount* account, const QString& jidFrom, const QString& body, const QString& subject);
     bool processOutgoingMessage(PsiAccount* account, const QString& jidTo, QString& body, const QString& type, QString& subject);
     void processOutgoingStanza(PsiAccount* account, QDomElement &stanza);
-    bool stanzaWasEncrypted(const QString &stanzaId);
     void logout(PsiAccount* account);
 
     void applyOptions(const QString& plugin);
@@ -95,6 +94,9 @@ public:
     QIcon icon(const QString& plugin) const;
     QStringList pluginFeatures() const;
 
+    bool decryptMessageElement(PsiAccount *account, QDomElement &message) const;
+    bool encryptMessageElement(PsiAccount *account, QDomElement &message) const;
+
     static const QString loadOptionPrefix;
     static const QString pluginOptionPrefix;
 
@@ -153,6 +155,8 @@ private:
     QString status(int account, const QString& jid) const;
     QString statusMessage(int account, const QString& jid) const;
     QStringList resources(int account, const QString& jid) const;
+    QString realJid(int account, const QString& jid) const;
+    QStringList mucNicks(int account, const QString& mucJid) const;
 
     bool setActivity(int account, const QString& Jid, QDomElement xml);
     bool setMood(int account, const QString& Jid, QDomElement xml);
@@ -168,11 +172,13 @@ private:
     void setStatus(int account, const QString& status, const QString& statusMessage);
 
     bool appendSysMsg(int account, const QString& jid, const QString& message);
-    bool appendMsg(int account, const QString& jid, const QString& message, const QString& id);
+    bool appendMsg(int account, const QString& jid, const QString& message, const QString& id, bool wasEncrypted);
 
     void createNewEvent(int account, const QString& jid, const QString& descr, QObject *receiver, const char* slot);
     void createNewMessageEvent(int account, QDomElement const &element);
 
+    void updateFeatures();
+
     friend class PluginHost;
 
 private slots:
