@@ -36,6 +36,9 @@
 #include "optionaccessor.h"
 #include "applicationinfoaccessor.h"
 #include "applicationinfoaccessinghost.h"
+#include "pluginaccessor.h"
+#include "pluginaccessinghost.h"
+#include "commandexecutor.h"
 #include <QByteArray>
 #include <QFile>
 #include <QFileDialog>
@@ -90,7 +93,8 @@ class HttpUploadPlugin: public QObject,
 		public OptionAccessor,
 		public ChatTabAccessor,
 		public StanzaFilter,
-		public ApplicationInfoAccessor {
+		public ApplicationInfoAccessor,
+		public PluginAccessor {
 Q_OBJECT
 #ifdef HAVE_QT5
 Q_PLUGIN_METADATA(IID "com.psi-plus.HttpUploadPlugin")
@@ -98,7 +102,7 @@ Q_PLUGIN_METADATA(IID "com.psi-plus.HttpUploadPlugin")
 Q_INTERFACES(PsiPlugin ToolbarIconAccessor GCToolbarIconAccessor
 		StanzaSender ActiveTabAccessor PsiAccountController OptionAccessor
 		IconFactoryAccessor AccountInfoAccessor PluginInfoProvider ChatTabAccessor
-		StanzaFilter ApplicationInfoAccessor)
+		StanzaFilter ApplicationInfoAccessor PluginAccessor)
 public:
 	HttpUploadPlugin();
 	virtual QString name() const;
@@ -124,6 +128,7 @@ public:
 	virtual void setAccountInfoAccessingHost(AccountInfoAccessingHost* host);
 	virtual void setPsiAccountControllingHost(PsiAccountControllingHost *host);
 	virtual void setOptionAccessingHost(OptionAccessingHost *host);
+	virtual void setPluginAccessingHost(PluginAccessingHost *host);
 	virtual void optionChanged(const QString &) {
 	}
 	virtual QString pluginInfo();
@@ -174,17 +179,13 @@ private:
 
 	void cancelTimeout() {
 		slotTimeout.stop();
-		if (dataSource) {
-			dataSource->deleteLater();
-		}
-		if (imageBytes) {
-			delete imageBytes;
-			imageBytes = 0;
-		}
+		dataSource = QByteArray();
 	}
 	void processServices(const QDomElement& query, int account);
 	void processOneService(const QDomElement& query, const QString& service, int account);
 	void processUploadSlot(const QDomElement& xml);
+	void omemoEncryptData();
+	QString omemoEncryptMessage(const QString &message);
 
 	IconFactoryAccessingHost* iconHost;
 	StanzaSendingHost* stanzaSender;
@@ -192,13 +193,12 @@ private:
 	AccountInfoAccessingHost* accInfo;
 	PsiAccountControllingHost *psiController;
 	OptionAccessingHost *psiOptions;
+	PluginAccessingHost *pluginHost;
 	ApplicationInfoAccessingHost* appInfoHost;
 	bool enabled;
-	QHash<QString, int> accounts_;
 	QNetworkAccessManager* manager;
 	QMap<QString, UploadService> serviceNames;
-	QPointer<QIODevice> dataSource;
-	QByteArray* imageBytes;
+	QByteArray dataSource;
 	CurrentUpload currentUpload;
 	QTimer slotTimeout;
 	QSpinBox *sb_previewWidth;
@@ -217,7 +217,7 @@ Q_EXPORT_PLUGIN(HttpUploadPlugin)
 
 HttpUploadPlugin::HttpUploadPlugin() :
 		iconHost(0), stanzaSender(0), activeTab(0), accInfo(0), psiController(0), psiOptions(0), appInfoHost(0), enabled(
-				false), manager(new QNetworkAccessManager(this)), imageBytes(0), sb_previewWidth(0), cb_resize(0), sb_size(
+				false), manager(new QNetworkAccessManager(this)), sb_previewWidth(0), cb_resize(0), sb_size(
 				0), sb_quality(0), imageResize(false), imageSize(0), imageQuality(0), previewWidth(0) {
 	connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(uploadComplete(QNetworkReply*)));
 	connect(&slotTimeout, SIGNAL(timeout()), this, SLOT(timeout()));
@@ -337,6 +337,10 @@ void HttpUploadPlugin::setOptionAccessingHost(OptionAccessingHost *host) {
 	psiOptions = host;
 }
 
+void HttpUploadPlugin::setPluginAccessingHost(PluginAccessingHost *host) {
+	pluginHost = host;
+}
+
 void HttpUploadPlugin::setStanzaSendingHost(StanzaSendingHost *host) {
 	stanzaSender = host;
 }
@@ -356,17 +360,13 @@ void HttpUploadPlugin::uploadImage() {
 void HttpUploadPlugin::upload(bool anything) {
 	if (!enabled)
 		return;
-	if (dataSource) {
+	if (!dataSource.isNull()) {
 		QMessageBox::warning(0, tr("Please wait"),
 				tr(
 						"Another upload operation is already in progress. Please wait up to %1 sec for it to complete or fail.").arg(
 				SLOT_TIMEOUT / 1000));
 		return;
 	}
-	if (imageBytes) {
-		delete imageBytes;
-		imageBytes = 0;
-	}
 	QString serviceName;
 	int sizeLimit = -1;
 	int account = accountNumber();
@@ -410,55 +410,52 @@ void HttpUploadPlugin::upload(bool anything) {
 	qDebug() << "MIME type:" << mimeType;
 #endif
 #endif
-	int length;
 	QString lowerImagename = imageName.toLower();
 	// only resize jpg and png
 	if (!anything && imageResize
 			&& (lowerImagename.endsWith(".jpg") || lowerImagename.endsWith(".jpeg") || lowerImagename.endsWith(".png"))
 			&& (pix.width() > imageSize || pix.height() > imageSize)) {
-		imageBytes = new QByteArray();
-		dataSource = new QBuffer(imageBytes);
 		QString type = "jpg";
 		if (lowerImagename.endsWith(".png")) {
 			type = "png";
 		}
-		pix.scaled(imageSize, imageSize, Qt::KeepAspectRatio, Qt::SmoothTransformation).save(dataSource,
+		QBuffer buffer(&dataSource);
+		pix.scaled(imageSize, imageSize, Qt::KeepAspectRatio, Qt::SmoothTransformation).save(&buffer,
 				type.toLatin1().constData(), imageQuality);
-		length = imageBytes->length();
 #ifdef DEBUG_UPLOAD
 		qDebug() << "Resized length:" << length;
 #endif
-		dataSource->open(QIODevice::ReadOnly);
 	} else {
-		length = fileInfo.size();
-		dataSource = new QFile(fileName, this);
-		if (!dataSource->open(QIODevice::ReadOnly)) {
-			dataSource->deleteLater();
+		if (!file.open(QIODevice::ReadOnly)) {
+			dataSource.clear();
 			QMessageBox::critical(0, tr("Error"), tr("Error opening file %1").arg(fileName));
 			return;
 		}
+		dataSource.resize(static_cast<int>(fileInfo.size()));
+		file.read(dataSource.data(), fileInfo.size());
 	}
-	if (length > sizeLimit) {
+	if (dataSource.length() > sizeLimit) {
 		QMessageBox::critical(0, tr("The file size is too large."),
 				tr("File size must be less than %1 bytes").arg(sizeLimit));
-		if (dataSource) {
-			dataSource->deleteLater();
-		}
+		dataSource = QByteArray();
 		return;
 	}
 	currentUpload.account = account;
 	currentUpload.from = jid;
 	currentUpload.to = jidToSend;
+	currentUpload.localFilePath = fileName;
 	currentUpload.type =
 			QLatin1String(sender()->parent()->metaObject()->className()) == "PsiChatDlg" ? "chat" : "groupchat";
 
+	omemoEncryptData();
+
 	QString slotRequestStanza = QString("<iq from='%1' id='%2' to='%3' type='get'>"
 			"<request xmlns='urn:xmpp:http:upload'>"
 			"<filename>%4</filename>"
 			"<size>%5</size>"
 			"<content-type>%6</content-type>"
 			"</request>"
-			"</iq>").arg(jid).arg(getId(account)).arg(serviceName).arg(escape(imageName)).arg(length).arg(mimeType);
+			"</iq>").arg(jid).arg(getId(account)).arg(serviceName).arg(escape(imageName)).arg(dataSource.size()).arg(mimeType);
 #ifdef DEBUG_UPLOAD
 	qDebug() << "Requesting slot:" << slotRequestStanza;
 #endif
@@ -580,13 +577,13 @@ void HttpUploadPlugin::processUploadSlot(const QDomElement& xml) {
 		currentUpload.getUrl = get;
 		QNetworkRequest req;
 		req.setUrl(QUrl(put));
-		if (!dataSource) {
+		if (dataSource.isNull()) {
 			QMessageBox::critical(0, tr("Error uploading"),
 					tr("No data to upload, this maybe a result of timeout or other error."));
 			cancelTimeout();
 			return;
 		}
-		qint64 size = dataSource->size();
+		qint64 size = dataSource.size();
 		req.setHeader(QNetworkRequest::ContentLengthHeader, size);
 		manager->put(req, dataSource);
 	}
@@ -610,6 +607,7 @@ bool HttpUploadPlugin::incomingStanza(int account, const QDomElement& xml) {
 }
 
 void HttpUploadPlugin::uploadComplete(QNetworkReply* reply) {
+	cancelTimeout();
 	bool ok;
 	int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt(&ok);
 	if (ok && (statusCode == 201 || statusCode == 200)) {
@@ -618,19 +616,42 @@ void HttpUploadPlugin::uploadComplete(QNetworkReply* reply) {
 				currentUpload.type == "chat"
 						&& psiOptions->getGlobalOption("options.ui.notifications.request-receipts").toBool() ?
 						"<request xmlns='urn:xmpp:receipts'/>" : "");
-		QString message = QString("<message type=\"%1\" to=\"%2\" id=\"%3\">"
-				"<body>%4</body>"
-				"%5"
-				"</message>").arg(currentUpload.type).arg(currentUpload.to).arg(id).arg(currentUpload.getUrl).arg(
-				receipt);
-		stanzaSender->sendStanza(currentUpload.account, message);
+		QString &url = currentUpload.getUrl;
+		bool omemoEncrypted = !currentUpload.aesgcmAnchor.isNull();
+		if (omemoEncrypted) {
+			url += "#" + currentUpload.aesgcmAnchor.toHex();
+			url.replace(QRegExp("https?://"), "aesgcm://");
+		}
+		QString message = QString(R"(<message type="%1" to="%2" id="%3">)");
+		if (!omemoEncrypted) {
+			message += R"(<x xmlns="jabber:x:oob">)"
+					   "<url>%4</url>"
+					   "</x>";
+		}
+		message += "<body>%4</body>"
+				   "%5"
+				   "</message>";
+		message = message.arg(currentUpload.type).arg(currentUpload.to).arg(id).arg(url).arg(receipt);
+		if (omemoEncrypted) {
+			message = omemoEncryptMessage(message);
+			if (message.isNull()) {
+				QMessageBox::critical(nullptr, tr("Error uploading"), tr("Something has gone wrong, please try again or disable OMEMO encryption"));
+				return;
+			}
+		}
+		// if the message is empty, then omemo will send the message asynchronously
+		if (!omemoEncrypted || !message.isEmpty()) {
+			stanzaSender->sendStanza(currentUpload.account, message);
+		}
 		if (currentUpload.type == "chat") {
 			// manually add outgoing message to the regular chats, in MUC this isn't needed
-			psiController->appendMsg(currentUpload.account, currentUpload.to, currentUpload.getUrl, id);
+			psiController->appendMsg(currentUpload.account, currentUpload.to, url, id, omemoEncrypted);
+			if (omemoEncrypted) {
+				psiController->appendMsg(currentUpload.account, currentUpload.to,
+								QUrl::fromLocalFile(currentUpload.localFilePath).toString(QUrl::FullyEncoded), id, omemoEncrypted);
+			}
 		}
-		cancelTimeout();
 	} else {
-		cancelTimeout();
 		QMessageBox::critical(0, tr("Error uploading"),
 				tr("Upload error %1; HTTP code %2, message: %3").arg(reply->errorString()).arg(
 						reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toString()).arg(
@@ -685,4 +706,35 @@ void HttpUploadPlugin::handleSslError(QNetworkReply* reply, const QList<QSslErro
 	reply->ignoreSslErrors();
 }
 
+void HttpUploadPlugin::omemoEncryptData() {
+	if (currentUpload.type != "chat") return;
+
+	CommandExecutor *plugin = qobject_cast<CommandExecutor*>(pluginHost->getPlugin("omemo"));
+	if (plugin == nullptr) return;
+
+	if (!plugin->execute(currentUpload.account, QHash<QString, QVariant>{{"is_enabled_for", currentUpload.to}})) {
+		return;
+	}
+
+	QHash<QString, QVariant> result;
+	if (!plugin->execute(currentUpload.account, QHash<QString, QVariant>{{"encrypt_data", dataSource}}, &result)) {
+		return;
+	}
+
+	dataSource.clear();
+	dataSource.insert(0, result["data"].toByteArray());
+	currentUpload.aesgcmAnchor = result["anchor"].toByteArray();
+}
+
+QString HttpUploadPlugin::omemoEncryptMessage(const QString &message) {
+	CommandExecutor *plugin = qobject_cast<CommandExecutor*>(pluginHost->getPlugin("omemo"));
+
+	QHash<QString, QVariant> result;
+	if (!plugin->execute(currentUpload.account, QHash<QString, QVariant>{{"encrypt_message", message}}, &result)) {
+		return QString();
+	}
+
+	return result.contains("message") ? result["message"].toString() : QString("");
+}
+
 #include "httpuploadplugin.moc"
