@@ -32,6 +32,7 @@
 
 #include "xmpp_rosterx.h"
 #include "xmpp_status.h"
+#include "xmpp_encryptionhandler.h"
 #include "psiactions.h"
 #include "psievent.h"
 #include "mood.h"
@@ -98,7 +99,7 @@ struct GCContact;
 
 class AvCallManager;
 
-class PsiAccount : public QObject
+class PsiAccount : public QObject, EncryptionHandler
 {
     Q_OBJECT
 protected:
@@ -166,7 +167,6 @@ public:
     const Mood &mood() const;
     const Activity &activity() const;
     const GeoLocation &geolocation() const;
-    void forceDisconnect(bool fast, const XMPP::Status &s);
     bool hasPGP() const;
     QHostAddress *localAddress() const;
 
@@ -209,6 +209,7 @@ public:
     void changePW();
     void changeStatus(int, bool forceDialog = false);
     void doDisco();
+    void doWakeup();
 
     void showXmlConsole();
     void openAddUserDlg();
@@ -272,7 +273,9 @@ public:
     bool ensureKey(const Jid &);
     void tryVerify(UserListItem *, UserResource *);
 
-    static void getErrorInfo(int err, AdvancedConnector *conn, Stream *stream, QCATLSHandler *tlsHandler, QString *_str, bool *_reconn, bool *_badPass, bool *_disableAutoConnect, bool *_isAuthError, bool *_isTemporaryAuthFailure);
+    static void getErrorInfo(int err, AdvancedConnector *conn, Stream *stream, QCATLSHandler *tlsHandler,
+                             QString *_str, bool *_reconn, bool *_badPass, bool *_disableAutoConnect,
+                             bool *_isTemporaryAuthFailure, bool *_needAlert);
 
     void deleteQueueFile();
     void sendFiles(const Jid&, const QStringList&, bool direct = false);
@@ -325,6 +328,7 @@ signals:
     void rosterRequestFinished();
 
 public slots:
+    void fastLogout();
     void setStatus(const XMPP::Status &, bool withPriority = false, bool isManualStatus = false);
     void showStatusDialog(const QString& presetName);
 
@@ -460,9 +464,7 @@ private slots:
 
     void incomingHttpAuthRequest(const PsiHttpAuthRequest &);
 
-    void reconnectOncePhase2();
     void reconnect();
-    void disconnect();
     void enableNotifyOnline();
 
 #ifdef WHITEBOARDING
@@ -560,6 +562,9 @@ private:
     QWidget* findDialog(const QMetaObject& mo, const Jid& jid, bool compareResource) const;
     void findDialogs(const QMetaObject& mo, const Jid& jid, bool compareResource, QList<void*>* list) const;
     void findDialogs(const QMetaObject& mo, QList<void*>* list) const;
+
+    bool decryptMessageElement(QDomElement &element) override;
+    bool encryptMessageElement(QDomElement &element) override;
 };
 
 #endif
