@@ -34,7 +34,10 @@
 #include <QImageReader>
 #include <QMessageBox>
 #include <QDir>
+#include <QSessionManager>
+#include <QNetworkConfigurationManager>
 
+#include "iris/processquit.h"
 #include "s5b.h"
 #include "xmpp_caps.h"
 #include "psiaccount.h"
@@ -109,6 +112,7 @@
 #include "contactupdatesmanager.h"
 #include "avcall/avcall.h"
 #include "avcall/calldlg.h"
+#include "avcall/mediadevicewatcher.h"
 #include "alertmanager.h"
 #include "bosskey.h"
 #include "popupmanager.h"
@@ -127,9 +131,6 @@
 #include "mac_dock/mac_dock.h"
 #endif
 
-// from opt_avcall.cpp
-extern void options_avcall_update();
-
 static const char *tunePublishOptionPath = "options.extended-presence.tune.publish";
 static const char *tuneUrlFilterOptionPath = "options.extended-presence.tune.url-filter";
 static const char *tuneTitleFilterOptionPath = "options.extended-presence.tune.title-filter";
@@ -225,6 +226,7 @@ public:
         , contactList(0)
         , iconSelect(0)
         , quitting(false)
+        , wakeupPending(false)
         , alertManager(parent)
         , bossKey(0)
         , popupManager(0)
@@ -305,11 +307,14 @@ public:
     QMenuBar* defaultMenuBar;
     TabManager *tabManager;
     bool quitting;
+    bool wakeupPending;
     QTimer* updatedAccountTimer_;
     AutoUpdater *autoUpdater;
     AlertManager alertManager;
     BossKey *bossKey;
     PopupManager * popupManager;
+    QNetworkConfigurationManager netConfMng;
+    QNetworkSession *netSession = nullptr;
 
     struct IdleSettings
     {
@@ -349,6 +354,7 @@ PsiCon::PsiCon()
     d = new Private(this);
     d->tabManager = new TabManager(this);
     connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), SLOT(aboutToQuit()));
+    connect(ProcessQuit::instance(), SIGNAL(quit()), SLOT(aboutToQuit()));
 
 
     d->mainwin = 0;
@@ -390,13 +396,20 @@ bool PsiCon::init()
     if (!ActiveProfiles::instance()->setThisProfile(activeProfile))
         return false;
 
-    connect(qApp, SIGNAL(forceSavePreferences()), SLOT(forceSavePreferences()));
+#if QT_VERSION >= QT_VERSION_CHECK(5,6,0)
+    qApp->setFallbackSessionManagementEnabled(false);
+#endif
+    connect(qApp, SIGNAL(commitDataRequest(QSessionManager&)), SLOT(forceSavePreferences(QSessionManager&)));
 
 #ifdef HAVE_PGPUTIL
     // PGP initialization (needs to be before any gpg usage!)
     PGPUtil::instance();
 #endif
 
+    d->netSession = new QNetworkSession(d->netConfMng.defaultConfiguration(), this);
+    connect(d->netSession, &QNetworkSession::opened, this, &PsiCon::networkSessionOpened);
+    d->netSession->open();
+
     d->contactList = new PsiContactList(this);
 
     connect(d->contactList, SIGNAL(accountAdded(PsiAccount*)), SIGNAL(accountAdded(PsiAccount*)));
@@ -667,10 +680,10 @@ bool PsiCon::init()
     DesktopUtil::setUrlHandler("x-psi-atstyle", this, "openAtStyleUri");
 
     if(AvCallManager::isSupported()) {
-        options_avcall_update();
-        AvCallManager::setAudioOutDevice(options->getOption("options.media.devices.audio-output").toString());
-        AvCallManager::setAudioInDevice(options->getOption("options.media.devices.audio-input").toString());
-        AvCallManager::setVideoInDevice(options->getOption("options.media.devices.video-input").toString());
+        auto config = MediaDeviceWatcher::instance()->configuration();
+        AvCallManager::setAudioOutDevice(config.audioOutDeviceId);
+        AvCallManager::setAudioInDevice(config.audioInDeviceId);
+        AvCallManager::setVideoInDevice(config.videoInDeviceId);
         AvCallManager::setBasePort(options->getOption("options.p2p.bytestreams.listen-port").toInt());
         AvCallManager::setExternalAddress(options->getOption("options.p2p.bytestreams.external-address").toString());
     }
@@ -713,7 +726,7 @@ void PsiCon::deinit()
     UserAccountList acc;
     if(d->contactList) {
         acc = d->contactList->getUserAccountList();
-        delete d->contactList;
+        delete d->contactList; // also deletes all accounts
     }
     d->nam->releaseHandlers();
 
@@ -744,6 +757,16 @@ void PsiCon::deinit()
     DesktopUtil::unsetUrlHandler("xmpp");
 }
 
+// will gracefully finish all network activity and other async stuff
+void PsiCon::gracefulDeinit(std::function<void ()> callback)
+{
+    if(d->contactList) {
+        connect(d->contactList, &PsiContactList::gracefulDeinitFinished, this, callback);
+        d->contactList->gracefulDeinit();
+    } else {
+        callback();
+    }
+}
 
 void PsiCon::setShortcuts()
 {
@@ -1027,6 +1050,7 @@ PsiAccount *PsiCon::createAccount(const UserAccount& _acc)
     connect(pa, SIGNAL(updatedAccount()), SLOT(pa_updatedAccount()));
     connect(pa, SIGNAL(queueChanged()), SLOT(queueChanged()));
     connect(pa, SIGNAL(startBounce()), SLOT(startBounce()));
+    connect(pa, SIGNAL(disconnected()), SLOT(proceedWithSleep()));
     if (d->s5bServer) {
         pa->client()->s5bManager()->setServer(d->s5bServer);
     }
@@ -1526,6 +1550,16 @@ void PsiCon::startBounce()
 #endif
 }
 
+void PsiCon::proceedWithSleep()
+{
+    foreach(PsiAccount* account, d->contactList->enabledAccounts()) {
+        if(account->loggedIn()) {
+            return; // we need all disconnedted to proceed with sleep
+        }
+    }
+    SystemWatch::instance()->proceedWithSleep();
+}
+
 void PsiCon::recvNextEvent()
 {
     /*printf("--- Queue Content: ---\n");
@@ -1874,18 +1908,19 @@ void PsiCon::doWakeup()
     // TODO: Restore the status from before the log out. Make it an (hidden) option for people with a bad wireless connection.
     //setGlobalStatus(Status());
 
-    foreach(PsiAccount* account, d->contactList->enabledAccounts()) {
-        if (account->userAccount().opt_connectAfterSleep) {
-            // Should we do this when the network comes up ?
-            if (account->accountOptions().opt_autoSameStatus) {
-                Status s = account->accountOptions().lastStatus;
-                account->setStatus(s, account->accountOptions().lastStatusWithPriority, true);
-                emit statusMessageChanged(s.status());
-            }
-            else {
-                account->setStatus(makeStatus(XMPP::Status::Online, ""), false, true);
-            }
+    d->wakeupPending = true; // and wait for signal till network session is opened (proved to work on gentoo+nm+xfce)
+}
+
+void PsiCon::networkSessionOpened()
+{
+    if (d->wakeupPending) {
+        d->wakeupPending = false;
+        foreach(PsiAccount* account, d->contactList->enabledAccounts()) {
+            account->doWakeup();
         }
+
+        if(d->contactList && d->contactList->defaultAccount())
+            emit statusMessageChanged(d->contactList->defaultAccount()->status().status());
     }
 }
 
@@ -1922,9 +1957,12 @@ QString PsiCon::optionsFile() const
     return pathToProfile(activeProfile, ApplicationInfo::ConfigLocation) + "/options.xml";
 }
 
-void PsiCon::forceSavePreferences()
+void PsiCon::forceSavePreferences(QSessionManager &session)
 {
+    session.setRestartHint(QSessionManager::RestartIfRunning);
     PsiOptions::instance()->save(optionsFile());
+    // TODO save any other options
+    // TODO warn about unfinished stuff like file transfer
 }
 
 void PsiCon::doQuit(int quitCode)
