@@ -22,6 +22,7 @@
 #define PSICON_H
 
 #include <QList>
+#include <functional>
 
 #include "profiles.h"
 #include "psiactions.h"
@@ -75,6 +76,7 @@ public:
 
     bool init();
     void deinit();
+    void gracefulDeinit(std::function<void()> callback);
 
     PsiContactList* contactList() const;
     EDB *edb() const;
@@ -179,11 +181,12 @@ public slots:
 private slots:
     void saveAccounts();
     void optionChanged(const QString& option);
-    void forceSavePreferences();
+    void forceSavePreferences(QSessionManager &);
     void startBounce();
     void aboutToQuit();
-    void secondsIdle(int);
-
+    void secondsIdle(int);    
+    void proceedWithSleep();
+    void networkSessionOpened();
 private:
     class Private;
     Private *d;
